#ifndef LIBTOF_H
#define LIBTOF_H

#define DEVELOP_DEBUG 0

#if WIN32
#  define Q_DECL_EXPORT __declspec(dllexport)
#  define Q_DECL_IMPORT __declspec(dllimport)
#else
#    define Q_DECL_EXPORT     __attribute__((visibility("default")))
#    define Q_DECL_IMPORT     __attribute__((visibility("default")))
#endif

#if defined(LIBTOF_LIBRARY)
#  define LIBTOFSHARED_EXPORT Q_DECL_EXPORT
#else
#  define LIBTOFSHARED_EXPORT Q_DECL_IMPORT
#endif

#if defined(_MSC_VER) && (_MSC_VER < 1600) && (!defined(_STDINT)) && (!defined(_STDINT_H))
typedef unsigned __int8   uint8_t;
typedef unsigned __int16  uint16_t;
typedef unsigned __int32  uint32_t;
typedef unsigned __int64  uint64_t;
#else
#include <stdint.h>
#endif

#if !defined(WIN32_WCE)
#include <sys/types.h>
#endif

#if defined(__linux) || defined(__APPLE__) || defined(__CYGWIN__) || defined(__HAIKU__)
#include <sys/time.h>
#endif

#include <time.h>
#include <limits.h>

#if WIN32
struct usb_dev_handle;
typedef struct usb_dev_handle usb_dev_handle;


#include <windows.h>
#else
typedef struct libusb_device_handle libusb_device_handle;
typedef struct libusb_context libusb_context;
typedef struct libusb_device libusb_device;
#endif


#include <stdio.h>
#include <string>
using namespace std;

#if defined(WIN32) || defined(__CYGWIN__) || defined(WIN32_WCE)
#define LIBTOF_CALL WINAPI
#else
#define LIBTOF_CALL
#endif

#ifndef NULL
#define NULL 0
#endif

typedef enum tag_ErrorCode_TOF
{
    /** Success (no error) */
    LTOF_SUCCESS = 0,
    /** Input/output error */
    LTOF_ERROR_IO = -1,
    /** Invalid parameter */
    LTOF_ERROR_INVALID_PARAM  = -2,
    /** Access denied (insufficient permissions) */
    LTOF_ERROR_ACCESS = -3,
    /** No such device (it may have been disconnected) */
    LTOF_ERROR_NO_DEVICE  =  -4,
    /** Operation timed out */
    LTOF_ERROR_TIMEOUT = -5,
    /** Overflow */
    LTOF_ERROR_OVERFLOW = -6,
    /** Insufficient memory */
    LTOF_ERROR_NO_MEM = -7,
    /** Operation not supported or unimplemented on this platform */
    LTOF_ERROR_NOT_SUPPORTED = -8,
    LTOF_ERROR_USB_WRITE = -9,
    LTOF_ERROR_USB_READ = -10,
    LTOF_ERROR_USB_DISCONNOT = -11,
    LTOF_ERROR_VERSION = -12,

    /** Other error */
    LTOF_ERROR_OTHER = -99,

}TOF_ErrorCode_t;
typedef struct tag_FRAMEDATA
{
    float        x;
    float        y;
    float        z;
}FrameData_t;
typedef struct tag_FRAMEDATA_RGB
{
    unsigned char        r;
    unsigned char        g;
    unsigned char        b;
}FrameDataRgb_t;
typedef struct tag_DeviceInfo
{
    uint32_t  DeviceVersion;
    uint32_t  DeviceType;
    uint16_t  DepthFrameWidth;
    uint16_t  DepthFrameHeight;
    uint16_t  BitsPerPoint;
    uint16_t  VisibleFrameWidth;
    uint16_t  VisibleFrameHeight;
    uint16_t  BitsPerPixel;
    uint16_t  BlockSizeIn;
    uint16_t BlockSizeOut;
    char    TofAlgVersion[20];
    char    DeviceId[19];
    uint8_t RgbdEn;
}DeviceInfo_t;
typedef enum CameraControlCommand
{
    /** Set the working mode of TOF camera,0：Mono-Frequency,1：Multi-Frequencies 2：GREY_SCALE */
    SET_CAMERA_MODE = (1<<16) |17,
    /** Command: Setting the frame rate */
    SET_CAMERA_FPS = (1<<16) |18,
    /** Command: Set the exposure time */
    SET_CAMERA_EXP = (1<<16) |19,
    /** Command: Set grey data  exposure time,range: 1-2000 */
    SET_GREY_EXP = 26,
}CameraControlCommand_e;

typedef enum SceneMode
{
    Scene_No  = 0,
    Scene_30cm  = 1,
    Scene_75cm  = 2,
}SceneMode_e;

typedef enum CameraUseCase
{
    /** Indoor room reconstruction*/
    MODE_1_5FPS_1200 = 0,
    /** Room scanning  indoor navigation*/
    MODE_2_10FPS_650,
    /** 3D object reconstruction*/
    MODE_3_15FPS_850,
    /** Medium size object recognition,face reconstruction*/
    MODE_4_30FPS_380,
    /** Remote collaboration,step by step instruction,table-top gaming*/
    MODE_5_45FPS_250,
}CameraUseCase_e;

typedef struct{
    int  version[3];         /*reserve byte*/
    int cameraNum;          /*camera number*/
    int width;                   /*image width*/
    int height;                  /*image height*/
    int  bpp;                      /*bpp  bits per pixel*/
    int dataFormat;          /*data format*/
    int frameCount;
    int getFrameTime;
    int sendFrameTime;
    int headerLength;
}S_MetaData;

typedef enum ExpourseMode
{
    Manual = 1,
    Auto = 2,
}ExpourseMode_e;

enum{
    MARS03_TOF = 1,
    MARS03_TOF_RGB,    //have rgb sensor
    MAR04_TOF,
    MARS04_TOF_RGB
};

typedef struct tag_IMUdata
{
    char nam[3];
    uint32_t timsstamp_ms;
    float accelData_x;
    float accelData_y;
    float accelData_z;

    float gyrData_x;
    float gyrData_y;
    float gyrData_z;

    float magData_x;
    float magData_y;
    float magData_z;
}IMU_data_t;

typedef struct tag_IMU_resv_data
{
    uint8_t imu_data_num;
    IMU_data_t imu_data[10];
}IMU_resv_data;

typedef struct tag_CmdBuffer
{
    uint16_t cmdlen;
    uint16_t size;
    unsigned char * buffer;
}CmdBuffer_t;

typedef struct tag_Camera_Parame
{
    char SerialNum[19];
    float Mtof[9];
    float Mrgb[9];
    float R[9];
    float T[3];
    char CRC16[2];
}Camera_Parame_t;

#if DEVELOP_DEBUG
class LIBTOFSHARED_EXPORT CLibTof
{
private:
    CLibTof();
#else
class CLibTof
{
public:
    CLibTof();
#endif

public:
    ~CLibTof();

public:
    static CLibTof* LibTOF_NewInstance();
    int  LibTOF_Connect();
    int  LibTOF_DisConnect();
    int  LibTOF_DepthCommand(CmdBuffer_t *CmdBuffer,CmdBuffer_t *RecvBuffer);
    int  LibTOF_RcvDepthFrame(FrameData_t* frame_data, FrameDataRgb_t *frame_data_Rgbd, 
                              unsigned char* frame_grey_data = NULL, unsigned char*fream_data_Raw = NULL,
							  unsigned char*fream_data_demarcate = NULL, short* frame_depth_raw = NULL, bool bDepthFile = false);
	int  LibTOF_RcvRgbFrame(short* rgb_data, unsigned int frame_len);
    int  LibTOF_GetTemperture(float* Cpu_t, float* Sensor_t);
    int  LibTOF_GetDeviceInfo(DeviceInfo_t *deviceinfo);
    int  LibTOF_UpdataFalsh(char* password);
    int  LibTOF_SetUseCase(CameraUseCase_e UseCase);
    int  LibTOF_SetSceneMode(SceneMode_e SceneMode);
    int  LibTOF_SetExpourseMode(ExpourseMode_e ExpMode);
    int  LibTOF_Setparam(CameraControlCommand_e Cmd, int value);
    int  LibTOF_SetGrayExp(int value);
	int  LibTOF_GetCameraParam(Camera_Parame_t &CameraParam);
	int  LibTOF_GetImuDate(IMU_resv_data* IMU_data);
	int  LibTOF_IsStartRaw(bool IsRaw);
	int  LibTOF_RgbdEnSet(int RgbdEn);

private:
#if WIN32
    usb_dev_handle	*mUsbHandle; /* opened device handle */
#else
    libusb_device_handle* mUsbHandle; /* opened device handle */
    libusb_context* mCtx;
    libusb_device** mDevs;
#endif

	DeviceInfo_t deviceinfo;
    int cur_scene;
    bool IS_running;
    float *gpLensparam;
    int cur_scence;
    int RgbdEnable;
    int camera_lastMode;  //Record the last mode 0: single frequency 1: double frequency.
    int camera_mode;      //记录灰度和rgbd模式的变量 0 rgbd  2 灰度
    bool camera_mode2;    //灰度模式下切换单双频变量  false 没有切换
    char serial_name[254];
    int  RcvDateFiledCount;
    int last_usecase;
    bool m_RecvDepthPause;
    bool m_RecvRgbPause;
    bool m_IsRawData;      //true Open access to raw data.
    bool m_IsRawVersion;   //true There is a version of raw data.
    bool m_M05Version;
    bool m_IsUpdata;
    bool m_Mtt015Movids;   //线性TOF 分支  15帧 双频的  
    float* m_AdiFilterBuf;
};

#endif // LIBTOF_H
