/*
 * opencvColorRead.cpp
 * Retrieves the color frame from the connected camera, and imports it into OpenCV for displaying.
 * Author: Cooper Lawrence, Metavues Tech
 * Last updated: 07/06/2019
 */

#include <MarsSDK.h>

#include <opencv2/core.hpp>
#include <opencv2/imgproc.hpp>
#include <opencv2/highgui.hpp>

int main()
{
	Mars* device = new Mars(); //Create a new Mars camera instance
	device->connect();

	ColorStream* colorStream = device->getColorStream(); //Create the camera's color stream

	//Create OpenCV Mat images to hold the YUV420 and BGR images
	cv::Mat yuv(colorStream->getHeight() + colorStream->getHeight() / 2, colorStream->getWidth(), CV_8UC1, colorStream->getFrame());
	cv::Mat bgr(colorStream->getHeight(), colorStream->getWidth(), CV_8UC3);

	//Load the color frame buffer from the camera
	while (cv::waitKey(1) < 0)
	{
		colorStream->update(); //Update the stream
		cv::cvtColor(yuv, bgr, cv::COLOR_YUV2BGR_I420); //Convert the YUV420 frame to BGR

		cv::imshow("Image", bgr); //Display the image
	}

	device->closeColorStream();
	device->disconnect();

	return 0;
}
