/*
 * simpleDepthRead.cpp
 * Retrieves the depth frame from the connected camera, and writes the depth to the console.
 * Author: Cooper Lawrence, Metavues Tech
 * Last updated: 12/08/2019
 */

#include <MarsSDK.h>

#include <chrono>

int main()
{
	Mars* device = new Mars(); //Create a new Mars camera instance
	device->connect();

	DepthStream* depthStream = device->getDepthStream(); //Create the camera's depth stream

	std::chrono::high_resolution_clock::time_point start = std::chrono::high_resolution_clock::now(); //Start a timer

	//Loop for 5 seconds
	while(std::chrono::duration_cast<std::chrono::seconds>(std::chrono::high_resolution_clock::now() - start).count() < 5)
	{
		depthStream->update(); //Update the depth frame
		
		printf("Depth: %f\n", depthStream->getFrame()[(depthStream->getSize() / 2) * 3 + 2]); //Print the depth at the centre of the camera. Pixel data can be accessed as (location * 3) + axis (where axis is 0 = x, 1 = y, 2 = z)
	}

	device->closeDepthStream();	
	device->disconnect();

	return 0;
}
